@@ -309,7 +309,7 @@ func TestPrepareFailureOnBuildError(t *testing.T) {
 	err := build.Run(&Config{}, &Trace{Writer: os.Stdout})
 
 	expectedErr := new(BuildError)
-	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
+	assert.ErrorIs(t, err, expectedErr)
 }
 
 func TestPrepareEnvironmentFailure(t *testing.T) {
@@ -346,7 +346,7 @@ func TestPrepareEnvironmentFailure(t *testing.T) {
 	}
 
 	err = build.Run(&Config{}, &Trace{Writer: os.Stdout})
-	assert.True(t, errors.Is(err, testErr))
+	assert.ErrorIs(t, err, testErr)
 }
 
 func TestJobFailure(t *testing.T) {
@@ -358,7 +358,7 @@ func TestJobFailure(t *testing.T) {
 	executor.On("Cleanup").Once()
 
 	// Set up a failing a build script
-	thrownErr := &BuildError{Inner: errors.New("test error")}
+	thrownErr := &BuildError{Inner: errors.New("test error"), ExitCode: 1}
 	executor.On("Shell").Return(&ShellScriptInfo{Shell: "script-shell"})
 	executor.On("Run", matchBuildStage(BuildStagePrepare)).Return(nil).Once()
 	executor.On("Run", mock.Anything).Return(thrownErr).Times(3)
@@ -385,12 +385,12 @@ func TestJobFailure(t *testing.T) {
 	trace.On("SetCancelFunc", mock.Anything).Once()
 	trace.On("SetAbortFunc", mock.Anything).Once()
 	trace.On("SetMasked", mock.Anything).Once()
-	trace.On("Fail", thrownErr, ScriptFailure).Once()
+	trace.On("Fail", thrownErr, JobFailureData{Reason: ScriptFailure, ExitCode: 1}).Once()
 
 	err = build.Run(&Config{}, trace)
 
 	expectedErr := new(BuildError)
-	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
+	assert.ErrorIs(t, err, expectedErr)
 }
 
 func TestJobFailureOnExecutionTimeout(t *testing.T) {
@@ -420,14 +420,14 @@ func TestJobFailureOnExecutionTimeout(t *testing.T) {
 	trace.On("SetCancelFunc", mock.Anything).Once()
 	trace.On("SetAbortFunc", mock.Anything).Once()
 	trace.On("SetMasked", mock.Anything).Once()
-	trace.On("Fail", mock.Anything, JobExecutionTimeout).Run(func(arguments mock.Arguments) {
+	trace.On("Fail", mock.Anything, JobFailureData{Reason: JobExecutionTimeout}).Run(func(arguments mock.Arguments) {
 		assert.Error(t, arguments.Get(0).(error))
 	}).Once()
 
 	err := build.Run(&Config{}, trace)
 
 	expectedErr := &BuildError{FailureReason: JobExecutionTimeout}
-	assert.True(t, errors.Is(err, expectedErr), "expected: %#v, got: %#v", expectedErr, err)
+	assert.ErrorIs(t, err, expectedErr)
 }
 
 func TestRunFailureRunsAfterScriptAndArtifactsOnFailure(t *testing.T) {
@@ -1605,7 +1605,7 @@ func TestBuild_GetExecutorJobSectionAttempts(t *testing.T) {
 			}
 
 			attempts, err := build.GetExecutorJobSectionAttempts()
-			assert.True(t, errors.Is(err, tt.expectedErr))
+			assert.ErrorIs(t, err, tt.expectedErr)
 			assert.Equal(t, tt.expectedAttempts, attempts)
 		})
 	}
@@ -1788,7 +1788,7 @@ func TestSecretsResolving(t *testing.T) {
 			assert.Equal(t, tt.expectedVariables, build.secretsVariables)
 
 			if tt.expectedError != nil {
-				assert.True(t, errors.As(err, &tt.expectedError))
+				assert.ErrorAs(t, err, &tt.expectedError)
 				return
 			}
 			assert.NoError(t, err)
