@@ -3,13 +3,14 @@ package buildtest
 import (
 	"bytes"
 	"os"
+	"strings"
 	"testing"
 	"time"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 )
 
-const testTimeout = 120 * time.Second
+const testTimeout = 30 * time.Minute
 
 func RunBuildReturningOutput(t *testing.T, build *common.Build) (string, error) {
 	buf := new(bytes.Buffer)
@@ -39,3 +40,28 @@ func RunBuild(t *testing.T, build *common.Build) error {
 
 	return err
 }
+
+// OnUserStage executes the provided function when the CurrentStage() enters
+// a non-predefined stage.
+func OnUserStage(build *common.Build, fn func()) func() {
+	exit := make(chan struct{})
+
+	go func() {
+		for {
+			select {
+			case <-exit:
+				return
+
+			case <-time.After(time.Second):
+				if strings.HasPrefix(string(build.CurrentStage()), "step_") {
+					fn()
+					return
+				}
+			}
+		}
+	}()
+
+	return func() {
+		close(exit)
+	}
+}
