@@ -32,6 +32,11 @@ const (
 	PullPolicyAlways       = "always"
 	PullPolicyNever        = "never"
 	PullPolicyIfNotPresent = "if-not-present"
+
+	DNSPolicyNone                    KubernetesDNSPolicy = "none"
+	DNSPolicyDefault                 KubernetesDNSPolicy = "default"
+	DNSPolicyClusterFirst            KubernetesDNSPolicy = "cluster-first"
+	DNSPolicyClusterFirstWithHostNet KubernetesDNSPolicy = "cluster-first-with-host-net"
 )
 
 // InvalidTimePeriodsError represents that the time period specified is not valid.
@@ -159,6 +164,7 @@ type ParallelsConfig struct {
 type VirtualBoxConfig struct {
 	BaseName         string `toml:"base_name" json:"base_name" long:"base-name" env:"VIRTUALBOX_BASE_NAME" description:"VM name to be used"`
 	BaseSnapshot     string `toml:"base_snapshot,omitempty" json:"base_snapshot" long:"base-snapshot" env:"VIRTUALBOX_BASE_SNAPSHOT" description:"Name or UUID of a specific VM snapshot to clone"`
+	BaseFolder       string `toml:"base_folder" json:"base_folder" long:"base-folder" env:"VIRTUALBOX_BASE_FOLDER" description:"Folder in which to save the new VM. If empty, uses VirtualBox default"`
 	DisableSnapshots bool   `toml:"disable_snapshots,omitzero" json:"disable_snapshots" long:"disable-snapshots" env:"VIRTUALBOX_DISABLE_SNAPSHOTS" description:"Disable snapshoting to speedup VM creation"`
 }
 
@@ -198,7 +204,31 @@ func (p KubernetesPullPolicy) Get() (KubernetesPullPolicy, error) {
 	case p == PullPolicyIfNotPresent:
 		return "IfNotPresent", nil
 	}
-	return "", fmt.Errorf("unsupported kubernetes-pull-policy: %v", p)
+	return "", fmt.Errorf("unsupported kubernetes-pull-policy %q", p)
+}
+
+type KubernetesDNSPolicy string
+
+// Get returns one of the predefined values in kubernetes notation or an error if the value is not matched.
+// If the DNSPolicy is a blank string, returns the k8s default ("ClusterFirst")
+func (p KubernetesDNSPolicy) Get() (api.DNSPolicy, error) {
+	const defaultPolicy = api.DNSClusterFirst
+
+	switch p {
+	case "":
+		logrus.Debugf("DNSPolicy string is blank, using %q as default", defaultPolicy)
+		return defaultPolicy, nil
+	case DNSPolicyNone:
+		return api.DNSNone, nil
+	case DNSPolicyDefault:
+		return api.DNSDefault, nil
+	case DNSPolicyClusterFirst:
+		return api.DNSClusterFirst, nil
+	case DNSPolicyClusterFirstWithHostNet:
+		return api.DNSClusterFirstWithHostNet, nil
+	}
+
+	return "", fmt.Errorf("unsupported kubernetes-dns-policy: %q", p)
 }
 
 //nolint:lll
@@ -266,9 +296,24 @@ type KubernetesConfig struct {
 	PodAnnotationsOverwriteAllowed                    string                       `toml:"pod_annotations_overwrite_allowed" json:"pod_annotations_overwrite_allowed" long:"pod_annotations_overwrite_allowed" env:"KUBERNETES_POD_ANNOTATIONS_OVERWRITE_ALLOWED" description:"Regex to validate 'KUBERNETES_POD_ANNOTATIONS_*' values"`
 	PodSecurityContext                                KubernetesPodSecurityContext `toml:"pod_security_context,omitempty" namespace:"pod-security-context" description:"A security context attached to each build pod"`
 	Volumes                                           KubernetesVolumes            `toml:"volumes"`
+	HostAliases                                       []KubernetesHostAliases      `toml:"host_aliases,omitempty" json:"host_aliases" long:"host_aliases" description:"Add a custom host-to-IP mapping"`
 	Services                                          []Service                    `toml:"services,omitempty" json:"services" description:"Add service that is started with container"`
 	CapAdd                                            []string                     `toml:"cap_add" json:"cap_add" long:"cap-add" env:"KUBERNETES_CAP_ADD" description:"Add Linux capabilities"`
 	CapDrop                                           []string                     `toml:"cap_drop" json:"cap_drop" long:"cap-drop" env:"KUBERNETES_CAP_DROP" description:"Drop Linux capabilities"`
+	DNSPolicy                                         KubernetesDNSPolicy          `toml:"dns_policy,omitempty" json:"dns_policy" long:"dns-policy" env:"KUBERNETES_DNS_POLICY" description:"How Kubernetes should try to resolve DNS from the created pods. If unset, Kubernetes will use the default 'ClusterFirst'. Valid values are: none, default, cluster-first, cluster-first-with-host-net"`
+	DNSConfig                                         KubernetesDNSConfig          `toml:"dns_config" json:"dns_config" description:"Pod DNS config"`
+}
+
+//nolint:lll
+type KubernetesDNSConfig struct {
+	Nameservers []string                    `toml:"nameservers" description:"A list of IP addresses that will be used as DNS servers for the Pod."`
+	Options     []KubernetesDNSConfigOption `toml:"options" description:"An optional list of objects where each object may have a name property (required) and a value property (optional)."`
+	Searches    []string                    `toml:"searches" description:"A list of DNS search domains for hostname lookup in the Pod."`
+}
+
+type KubernetesDNSConfigOption struct {
+	Name  string  `toml:"name"`
+	Value *string `toml:"value,omitempty"`
 }
 
 type KubernetesVolumes struct {
@@ -342,6 +387,12 @@ type KubernetesNodeAffinity struct {
 	PreferredDuringSchedulingIgnoredDuringExecution []PreferredSchedulingTerm `toml:"preferred_during_scheduling_ignored_during_execution,omitempty" json:"preferred_during_scheduling_ignored_during_execution"`
 }
 
+//nolint:lll
+type KubernetesHostAliases struct {
+	IP        string   `toml:"ip" json:"ip" long:"ip" description:"The IP address you want to attach hosts to"`
+	Hostnames []string `toml:"hostnames" json:"hostnames" long:"hostnames" description:"A list of hostnames that will be attached to the IP"`
+}
+
 type NodeSelector struct {
 	NodeSelectorTerms []NodeSelectorTerm `toml:"node_selector_terms" json:"node_selector_terms"`
 }
@@ -668,6 +719,26 @@ func (c *KubernetesConfig) GetAffinity() *api.Affinity {
 	return &affinity
 }
 
+func (c *KubernetesConfig) GetDNSConfig() *api.PodDNSConfig {
+	if len(c.DNSConfig.Nameservers) == 0 && len(c.DNSConfig.Searches) == 0 && len(c.DNSConfig.Options) == 0 {
+		return nil
+	}
+
+	var config api.PodDNSConfig
+
+	config.Nameservers = c.DNSConfig.Nameservers
+	config.Searches = c.DNSConfig.Searches
+
+	for _, opt := range c.DNSConfig.Options {
+		config.Options = append(config.Options, api.PodDNSConfigOption{
+			Name:  opt.Name,
+			Value: opt.Value,
+		})
+	}
+
+	return &config
+}
+
 //nolint:lll
 func (c *KubernetesConfig) GetNodeAffinity() *api.NodeAffinity {
 	var nodeAffinity api.NodeAffinity
@@ -717,6 +788,22 @@ func (c *PreferredSchedulingTerm) GetPreferredSchedulingTerm() api.PreferredSche
 	}
 }
 
+func (c *KubernetesConfig) GetHostAliases() []api.HostAlias {
+	var hostAliases []api.HostAlias
+
+	for _, hostAlias := range c.HostAliases {
+		hostAliases = append(
+			hostAliases,
+			api.HostAlias{
+				IP:        hostAlias.IP,
+				Hostnames: hostAlias.Hostnames,
+			},
+		)
+	}
+
+	return hostAliases
+}
+
 func (c *DockerMachine) GetIdleCount() int {
 	autoscaling := c.getActiveAutoscalingConfig()
 	if autoscaling != nil {
