@@ -1,7 +1,6 @@
 package common
 
 import (
-	"errors"
 	"fmt"
 	"testing"
 	"time"
@@ -9,6 +8,7 @@ import (
 	"github.com/BurntSushi/toml"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	api "k8s.io/api/core/v1"
 
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/process"
 )
@@ -305,6 +305,90 @@ func TestConfigParse(t *testing.T) {
 				assert.Nil(t, config.Runners[0].ForceKillTimeout)
 			},
 		},
+		"setting DNS policy to none": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						dns_policy = 'none'
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				dnsPolicy, err := config.Runners[0].Kubernetes.DNSPolicy.Get()
+				assert.NoError(t, err)
+				assert.Equal(t, api.DNSNone, dnsPolicy)
+			},
+		},
+		"setting DNS policy to default": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						dns_policy = 'default'
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				dnsPolicy, err := config.Runners[0].Kubernetes.DNSPolicy.Get()
+				assert.NoError(t, err)
+				assert.Equal(t, api.DNSDefault, dnsPolicy)
+			},
+		},
+		"setting DNS policy to cluster-first": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						dns_policy = 'cluster-first'
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				dnsPolicy, err := config.Runners[0].Kubernetes.DNSPolicy.Get()
+				assert.NoError(t, err)
+				assert.Equal(t, api.DNSClusterFirst, dnsPolicy)
+			},
+		},
+		"setting DNS policy to cluster-first-with-host-net": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						dns_policy = 'cluster-first-with-host-net'
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				dnsPolicy, err := config.Runners[0].Kubernetes.DNSPolicy.Get()
+				assert.NoError(t, err)
+				assert.Equal(t, api.DNSClusterFirstWithHostNet, dnsPolicy)
+			},
+		},
+		"fail setting DNS policy to invalid value": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						dns_policy = 'some-invalid-policy'
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				dnsPolicy, err := config.Runners[0].Kubernetes.DNSPolicy.Get()
+				assert.Error(t, err)
+				assert.Empty(t, dnsPolicy)
+			},
+		},
+		"fail setting DNS policy to empty value returns default value": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						dns_policy = ''
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				dnsPolicy, err := config.Runners[0].Kubernetes.DNSPolicy.Get()
+				assert.NoError(t, err)
+				assert.Equal(t, api.DNSClusterFirst, dnsPolicy)
+			},
+		},
 	}
 
 	for tn, tt := range tests {
@@ -324,6 +408,96 @@ func TestConfigParse(t *testing.T) {
 	}
 }
 
+func TestKubernetesHostAliases(t *testing.T) {
+	tests := map[string]struct {
+		config              KubernetesConfig
+		expectedHostAliases []api.HostAlias
+	}{
+		"parse Kubernetes HostAliases with empty list": {
+			config:              KubernetesConfig{},
+			expectedHostAliases: nil,
+		},
+		"parse Kubernetes HostAliases with unique ips": {
+			config: KubernetesConfig{
+				HostAliases: []KubernetesHostAliases{
+					{
+						IP:        "127.0.0.1",
+						Hostnames: []string{"web1", "web2"},
+					},
+					{
+						IP:        "192.168.1.1",
+						Hostnames: []string{"web14", "web15"},
+					},
+				},
+			},
+			expectedHostAliases: []api.HostAlias{
+				{
+					IP:        "127.0.0.1",
+					Hostnames: []string{"web1", "web2"},
+				},
+				{
+					IP:        "192.168.1.1",
+					Hostnames: []string{"web14", "web15"},
+				},
+			},
+		},
+		"parse Kubernetes HostAliases with duplicated ip": {
+			config: KubernetesConfig{
+				HostAliases: []KubernetesHostAliases{
+					{
+						IP:        "127.0.0.1",
+						Hostnames: []string{"web1", "web2"},
+					},
+					{
+						IP:        "127.0.0.1",
+						Hostnames: []string{"web14", "web15"},
+					},
+				},
+			},
+			expectedHostAliases: []api.HostAlias{
+				{
+					IP:        "127.0.0.1",
+					Hostnames: []string{"web1", "web2"},
+				},
+				{
+					IP:        "127.0.0.1",
+					Hostnames: []string{"web14", "web15"},
+				},
+			},
+		},
+		"parse Kubernetes HostAliases with duplicated hostname": {
+			config: KubernetesConfig{
+				HostAliases: []KubernetesHostAliases{
+					{
+						IP:        "127.0.0.1",
+						Hostnames: []string{"web1", "web1", "web2"},
+					},
+					{
+						IP:        "127.0.0.1",
+						Hostnames: []string{"web1", "web15"},
+					},
+				},
+			},
+			expectedHostAliases: []api.HostAlias{
+				{
+					IP:        "127.0.0.1",
+					Hostnames: []string{"web1", "web1", "web2"},
+				},
+				{
+					IP:        "127.0.0.1",
+					Hostnames: []string{"web1", "web15"},
+				},
+			},
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			assert.Equal(t, tt.expectedHostAliases, tt.config.GetHostAliases())
+		})
+	}
+}
+
 func TestService_ToImageDefinition(t *testing.T) {
 	tests := map[string]struct {
 		service       Service
@@ -533,7 +707,7 @@ func TestDockerMachine(t *testing.T) {
 		t.Run(tn, func(t *testing.T) {
 			err := tt.config.CompilePeriods()
 			if tt.expectedErr != nil {
-				assert.True(t, errors.Is(err, tt.expectedErr))
+				assert.ErrorIs(t, err, tt.expectedErr)
 				return
 			}
 			assert.NoError(t, err, "should not return err on good period compile")
