@@ -83,6 +83,7 @@ type FeaturesInfo struct {
 	MultiBuildSteps         bool `json:"multi_build_steps"`
 	TraceReset              bool `json:"trace_reset"`
 	TraceChecksum           bool `json:"trace_checksum"`
+	TraceSize               bool `json:"trace_size"`
 	VaultSecrets            bool `json:"vault_secrets"`
 	Cancelable              bool `json:"cancelable"`
 }
@@ -452,7 +453,14 @@ type UpdateJobRequest struct {
 	Token         string           `json:"token,omitempty"`
 	State         JobState         `json:"state,omitempty"`
 	FailureReason JobFailureReason `json:"failure_reason,omitempty"`
-	Checksum      string           `json:"checksum,omitempty"`
+	Checksum      string           `json:"checksum,omitempty"` // deprecated
+	Output        JobTraceOutput   `json:"output,omitempty"`
+	ExitCode      int              `json:"exit_code,omitempty"`
+}
+
+type JobTraceOutput struct {
+	Checksum string `json:"checksum,omitempty"`
+	Bytesize int    `json:"bytesize,omitempty"`
 }
 
 //nolint:lll
@@ -489,7 +497,8 @@ type UpdateJobInfo struct {
 	ID            int
 	State         JobState
 	FailureReason JobFailureReason
-	Checksum      string
+	Output        JobTraceOutput
+	ExitCode      int
 }
 
 type ArtifactsOptions struct {
@@ -506,7 +515,7 @@ type FailuresCollector interface {
 type JobTrace interface {
 	io.Writer
 	Success()
-	Fail(err error, failureReason JobFailureReason)
+	Fail(err error, failureData JobFailureData)
 	SetCancelFunc(cancelFunc context.CancelFunc)
 	Cancel() bool
 	SetAbortFunc(abortFunc context.CancelFunc)
