@@ -26,6 +26,7 @@ import (
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/executors"
+	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/exec"
 	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/labels"
 	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/networks"
 	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/volumes"
@@ -101,7 +102,7 @@ func init() {
 	if err != nil {
 		logrus.Errorln(
 			"Docker executor: unable to detect gitlab-runner folder, "+
-				"prebuilt image helpers will be loaded from DockerHub.",
+				"prebuilt image helpers will be loaded from remote registry.",
 			err,
 		)
 	}
@@ -291,7 +292,11 @@ func (e *executor) getPrebuiltImage() (*types.ImageInspect, error) {
 		return loadedImage, nil
 	}
 
-	// Fallback to getting image from DockerHub
+	if !e.Build.IsFeatureFlagOn(featureflags.GitLabRegistryHelperImage) {
+		e.Warningln(helperimage.DockerHubWarningMessage)
+	}
+
+	// Fall back to getting image from registry
 	e.Debugln(fmt.Sprintf("Loading image form registry: %s", e.helperImageInfo))
 	return e.getDockerImage(e.helperImageInfo.String())
 }
@@ -307,7 +312,11 @@ func (e *executor) getLocalHelperImage() *types.ImageInspect {
 			dockerPrebuiltImagesPath,
 			"prebuilt-"+architecture+prebuiltImageExtension,
 		)
-		image, err := e.loadPrebuiltImage(dockerPrebuiltImageFilePath, prebuiltImageName, e.helperImageInfo.Tag)
+		image, err := e.loadPrebuiltImage(
+			dockerPrebuiltImageFilePath,
+			e.helperImageInfo.Name,
+			e.helperImageInfo.Tag,
+		)
 		if err != nil {
 			e.Debugln("Failed to load prebuilt image from:", dockerPrebuiltImageFilePath, "error:", err)
 			continue
@@ -799,62 +808,9 @@ func (e *executor) createHostConfig() (*container.HostConfig, error) {
 }
 
 func (e *executor) startAndWatchContainer(ctx context.Context, id string, input io.Reader) error {
-	options := types.ContainerAttachOptions{
-		Stream: true,
-		Stdin:  true,
-		Stdout: true,
-		Stderr: true,
-	}
+	dockerExec := exec.NewDocker(e.client, e.waiter, e.Build.Log())
 
-	e.Debugln("Attaching to container", id, "...")
-	hijacked, err := e.client.ContainerAttach(ctx, id, options)
-	if err != nil {
-		return err
-	}
-	defer hijacked.Close()
-
-	e.Debugln("Starting container", id, "...")
-	err = e.client.ContainerStart(ctx, id, types.ContainerStartOptions{})
-	if err != nil {
-		return err
-	}
-
-	// Copy any output to the build trace
-	stdoutErrCh := make(chan error)
-	go func() {
-		_, errCopy := stdcopy.StdCopy(e.Trace, e.Trace, hijacked.Reader)
-		stdoutErrCh <- errCopy
-	}()
-
-	// Write the input to the container and close its STDIN to get it to finish
-	stdinErrCh := make(chan error)
-	go func() {
-		_, errCopy := io.Copy(hijacked.Conn, input)
-		_ = hijacked.CloseWrite()
-		if errCopy != nil {
-			stdinErrCh <- errCopy
-		}
-	}()
-
-	// Wait until either:
-	// - the job is aborted/cancelled/deadline exceeded
-	// - stdin has an error
-	// - stdout returns an error or nil, indicating the stream has ended and
-	//   the container has exited
-	select {
-	case <-ctx.Done():
-		err = errors.New("aborted")
-	case err = <-stdinErrCh:
-	case err = <-stdoutErrCh:
-	}
-
-	if err != nil {
-		e.Debugln("Container", id, "finished with", err)
-	}
-
-	// Kill and wait for exit.
-	// Containers are stopped so that they can be reused by the job.
-	return e.waiter.KillWait(ctx, id)
+	return dockerExec.Exec(ctx, id, input, e.Trace)
 }
 
 func (e *executor) removeContainer(ctx context.Context, id string) error {
@@ -940,8 +896,8 @@ func (e *executor) verifyAllowedImage(image, optionName string, allowedImages, i
 	}
 
 	e.Println(
-		"Please check runner's configuration: " +
-			"http://doc.gitlab.com/ci/docker/using_docker_images.html#overwrite-image-and-services")
+		"Please check runner's configuration allowed_images and allowed_services: " +
+			"https://docs.gitlab.com/runner/configuration/advanced-configuration.html#the-runnersdocker-section")
 	return errors.New("invalid image")
 }
 
@@ -992,12 +948,6 @@ func (e *executor) connectDocker() error {
 		return err
 	}
 
-	e.helperImageInfo, err = helperimage.Get(common.REVISION, helperimage.Config{
-		OSType:          e.info.OSType,
-		Architecture:    e.info.Architecture,
-		OperatingSystem: e.info.OperatingSystem,
-		Shell:           e.Config.Shell,
-	})
 	e.waiter = wait.NewDockerKillWaiter(e.client)
 
 	return err
@@ -1118,6 +1068,11 @@ func (e *executor) Prepare(options common.ExecutorPrepareOptions) error {
 		return err
 	}
 
+	e.helperImageInfo, err = e.prepareHelperImage()
+	if err != nil {
+		return err
+	}
+
 	err = e.prepareBuildsDir(options)
 	if err != nil {
 		return err
@@ -1146,6 +1101,16 @@ func (e *executor) Prepare(options common.ExecutorPrepareOptions) error {
 	return nil
 }
 
+func (e *executor) prepareHelperImage() (helperimage.Info, error) {
+	return helperimage.Get(common.REVISION, helperimage.Config{
+		OSType:          e.info.OSType,
+		Architecture:    e.info.Architecture,
+		OperatingSystem: e.info.OperatingSystem,
+		Shell:           e.Config.Shell,
+		GitLabRegistry:  e.Build.IsFeatureFlagOn(featureflags.GitLabRegistryHelperImage),
+	})
+}
+
 func (e *executor) prepareBuildsDir(options common.ExecutorPrepareOptions) error {
 	if e.volumeParser == nil {
 		return common.MakeBuildError("missing volume parser")
