@@ -141,13 +141,7 @@ func TestDefaultManager_CreateUserVolumes_HostVolume(t *testing.T) {
 			}
 
 			err = m.Create(context.Background(), testCase.volume)
-			assert.True(
-				t,
-				errors.Is(err, testCase.expectedError),
-				"expected err %T, but got %T",
-				testCase.expectedError,
-				err,
-			)
+			assert.ErrorIs(t, err, testCase.expectedError)
 			assert.Equal(t, testCase.expectedBinding, m.volumeBindings)
 		})
 	}
@@ -223,13 +217,7 @@ func TestDefaultManager_CreateUserVolumes_CacheVolume_Disabled(t *testing.T) {
 			}
 
 			err = m.Create(context.Background(), testCase.volume)
-			assert.True(
-				t,
-				errors.Is(err, testCase.expectedError),
-				"expected err %T, but got %T",
-				testCase.expectedError,
-				err,
-			)
+			assert.ErrorIs(t, err, testCase.expectedError)
 			assert.Equal(t, expectedBinding, m.volumeBindings)
 		})
 	}
@@ -319,13 +307,7 @@ func TestDefaultManager_CreateUserVolumes_CacheVolume_HostBased(t *testing.T) {
 				Once()
 
 			err = m.Create(context.Background(), testCase.volume)
-			assert.True(
-				t,
-				errors.Is(err, testCase.expectedError),
-				"expected err %T, but got %T",
-				testCase.expectedError,
-				err,
-			)
+			assert.ErrorIs(t, err, testCase.expectedError)
 			assert.Equal(t, testCase.expectedBinding, m.volumeBindings)
 		})
 	}
@@ -418,13 +400,7 @@ func TestDefaultManager_CreateUserVolumes_CacheVolume_VolumeBased(t *testing.T)
 
 			err = m.Create(context.Background(), testCase.volume)
 			if testCase.expectedError != nil {
-				assert.True(
-					t,
-					errors.Is(err, testCase.expectedError),
-					"expected err %T, but got %T",
-					testCase.expectedError,
-					err,
-				)
+				assert.ErrorIs(t, err, testCase.expectedError)
 				return
 			}
 
@@ -466,7 +442,7 @@ func TestDefaultManager_CreateUserVolumes_CacheVolume_VolumeBased_WithError(t *t
 		Once()
 
 	err := m.Create(context.Background(), "volume")
-	assert.True(t, errors.Is(err, testErr), "expected err %T, but got %T", testErr, err)
+	assert.ErrorIs(t, err, testErr)
 }
 
 func TestDefaultManager_CreateUserVolumes_ParserError(t *testing.T) {
@@ -482,7 +458,7 @@ func TestDefaultManager_CreateUserVolumes_ParserError(t *testing.T) {
 		Once()
 
 	err := m.Create(context.Background(), "volume")
-	assert.True(t, errors.Is(err, testErr), "expected err %T, but got %T", testErr, err)
+	assert.ErrorIs(t, err, testErr)
 }
 
 func TestDefaultManager_CreateTemporary(t *testing.T) {
@@ -560,25 +536,13 @@ func TestDefaultManager_CreateTemporary(t *testing.T) {
 
 			err = m.CreateTemporary(context.Background(), testCase.volume)
 			if testCase.expectedError != nil {
-				assert.True(
-					t,
-					errors.Is(err, testCase.expectedError),
-					"expected err %T, but got %T",
-					testCase.expectedError,
-					err,
-				)
+				assert.ErrorIs(t, err, testCase.expectedError)
 				return
 			}
 
 			require.Len(t, m.temporaryVolumes, 1)
 			assert.Equal(t, m.temporaryVolumes[0], testCase.expectedVolumeName)
-			assert.True(
-				t,
-				errors.Is(err, testCase.expectedError),
-				"expected err %T, but got %T",
-				testCase.expectedError,
-				err,
-			)
+			assert.ErrorIs(t, err, testCase.expectedError)
 			assert.Equal(t, testCase.expectedBindings, m.Binds())
 		})
 	}
@@ -633,13 +597,7 @@ func TestDefaultManager_RemoveTemporary(t *testing.T) {
 			m.temporaryVolumes = testCase.temporaryVolumes
 
 			err := m.RemoveTemporary(context.Background())
-			assert.True(
-				t,
-				errors.Is(err, testCase.expectedError),
-				"expected err %T, but got %T",
-				testCase.expectedError,
-				err,
-			)
+			assert.ErrorIs(t, err, testCase.expectedError)
 		})
 	}
 }
