@@ -12,7 +12,6 @@ import (
 
 	"github.com/docker/docker/api/types"
 	"github.com/jpillora/backoff"
-	"github.com/sirupsen/logrus"
 	"golang.org/x/net/context"
 	api "k8s.io/api/core/v1"
 	kubeerrors "k8s.io/apimachinery/pkg/api/errors"
@@ -21,11 +20,11 @@ import (
 	"k8s.io/client-go/kubernetes"
 	_ "k8s.io/client-go/plugin/pkg/client/auth" // Register all available authentication methods
 	restclient "k8s.io/client-go/rest"
+	"k8s.io/client-go/util/exec"
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/executors"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/container/helperimage"
-	"gitlab.com/gitlab-org/gitlab-runner/helpers/container/services"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/dns"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker/auth"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
@@ -163,6 +162,11 @@ func (s *executor) Prepare(options common.ExecutorPrepareOptions) (err error) {
 		return fmt.Errorf("connecting to Kubernetes: %w", err)
 	}
 
+	s.helperImageInfo, err = s.prepareHelperImage()
+	if err != nil {
+		return fmt.Errorf("prepare helper image: %w", err)
+	}
+
 	s.featureChecker = &kubeClientFeatureChecker{kubeClient: s.kubeClient}
 
 	s.Println("Using Kubernetes executor with image", s.options.Image.Name, "...")
@@ -173,6 +177,18 @@ func (s *executor) Prepare(options common.ExecutorPrepareOptions) (err error) {
 	return nil
 }
 
+func (s *executor) prepareHelperImage() (helperimage.Info, error) {
+	if !s.Build.IsFeatureFlagOn(featureflags.GitLabRegistryHelperImage) {
+		s.Warningln(helperimage.DockerHubWarningMessage)
+	}
+
+	return helperimage.Get(common.REVISION, helperimage.Config{
+		OSType:         helperimage.OSTypeLinux,
+		Architecture:   "amd64",
+		GitLabRegistry: s.Build.IsFeatureFlagOn(featureflags.GitLabRegistryHelperImage),
+	})
+}
+
 func (s *executor) Run(cmd common.ExecutorCommand) error {
 	if s.Build.IsFeatureFlagOn(featureflags.UseLegacyKubernetesExecutionStrategy) {
 		s.Debugln("Starting Kubernetes command...")
@@ -216,8 +232,9 @@ func (s *executor) runWithExecLegacy(cmd common.ExecutorCommand) error {
 	select {
 	case err := <-s.runInContainerWithExecLegacy(ctx, containerName, containerCommand, cmd.Script):
 		s.Debugln(fmt.Sprintf("Container %q exited with error: %v", containerName, err))
-		if err != nil && strings.Contains(err.Error(), "command terminated with exit code") {
-			return &common.BuildError{Inner: err}
+		var exitError exec.CodeExitError
+		if err != nil && errors.As(err, &exitError) {
+			return &common.BuildError{Inner: err, ExitCode: exitError.ExitStatus()}
 		}
 		return err
 
@@ -268,8 +285,9 @@ func (s *executor) runWithAttach(cmd common.ExecutorCommand) error {
 	select {
 	case err := <-s.runInContainer(containerName, containerCommand):
 		s.Debugln(fmt.Sprintf("Container %q exited with error: %v", containerName, err))
-		if err != nil && errors.Is(err, new(commandTerminatedError)) {
-			return &common.BuildError{Inner: err}
+		var terminatedError *commandTerminatedError
+		if err != nil && errors.As(err, &terminatedError) {
+			return &common.BuildError{Inner: err, ExitCode: terminatedError.exitCode}
 		}
 
 		return err
@@ -322,7 +340,7 @@ func (s *executor) buildLogPermissionsInitContainer() api.Container {
 
 	return api.Container{
 		Name:            "change-logs-permissions",
-		Image:           "busybox",
+		Image:           s.helperImageInfo.String(),
 		Command:         []string{"sh", "-c", chmod},
 		VolumeMounts:    s.getVolumeMounts(),
 		ImagePullPolicy: api.PullPolicy(s.pullPolicy),
@@ -818,33 +836,16 @@ func (s *executor) setupCredentials() error {
 	secret.Data = map[string][]byte{}
 	secret.Data[api.DockerConfigKey] = dockerCfgContent
 
-	s.credentials, err = s.kubeClient.CoreV1().Secrets(s.configurationOverwrites.namespace).Create(&secret)
+	creds, err := s.kubeClient.CoreV1().Secrets(s.configurationOverwrites.namespace).Create(&secret)
 	if err != nil {
 		return err
 	}
+
+	s.credentials = creds
 	return nil
 }
 
-type invalidHostAliasDNSError struct {
-	service common.Image
-	inner   error
-}
-
-func (e *invalidHostAliasDNSError) Error() string {
-	return fmt.Sprintf(
-		"provided host alias %s for service %s is invalid DNS. %s",
-		e.service.Alias,
-		e.service.Name,
-		e.inner,
-	)
-}
-
-func (e *invalidHostAliasDNSError) Is(err error) bool {
-	_, ok := err.(*invalidHostAliasDNSError)
-	return ok
-}
-
-func (s *executor) prepareHostAlias() (*api.HostAlias, error) {
+func (s *executor) getHostAliases() ([]api.HostAlias, error) {
 	supportsHostAliases, err := s.featureChecker.IsHostAliasSupported()
 	switch {
 	case errors.Is(err, &badVersionError{}):
@@ -856,44 +857,7 @@ func (s *executor) prepareHostAlias() (*api.HostAlias, error) {
 		return nil, nil
 	}
 
-	return s.createHostAlias()
-}
-
-func (s *executor) createHostAlias() (*api.HostAlias, error) {
-	servicesHostAlias := api.HostAlias{IP: "127.0.0.1"}
-
-	for _, service := range s.options.Services {
-		// Services with ports are coming from .gitlab-webide.yml
-		// they are used for ports mapping and their aliases are in no way validated
-		// so we ignore them. Check out https://gitlab.com/gitlab-org/gitlab-runner/merge_requests/1170
-		// for details
-		if len(service.Ports) > 0 {
-			continue
-		}
-
-		serviceMeta := services.SplitNameAndVersion(service.Name)
-		for _, alias := range serviceMeta.Aliases {
-			// For backward compatibility reasons a non DNS1123 compliant alias might be generated,
-			// this will be removed in https://gitlab.com/gitlab-org/gitlab-runner/issues/6100
-			err := dns.ValidateDNS1123Subdomain(alias)
-			if err == nil {
-				servicesHostAlias.Hostnames = append(servicesHostAlias.Hostnames, alias)
-			}
-		}
-
-		if service.Alias == "" {
-			continue
-		}
-
-		err := dns.ValidateDNS1123Subdomain(service.Alias)
-		if err != nil {
-			return nil, &invalidHostAliasDNSError{service: service, inner: err}
-		}
-
-		servicesHostAlias.Hostnames = append(servicesHostAlias.Hostnames, service.Alias)
-	}
-
-	return &servicesHostAlias, nil
+	return createHostAliases(s.options.Services, s.Config.Kubernetes.GetHostAliases())
 }
 
 func (s *executor) setupBuildPod(initContainers []api.Container) error {
@@ -933,12 +897,12 @@ func (s *executor) setupBuildPod(initContainers []api.Container) error {
 		imagePullSecrets = append(imagePullSecrets, api.LocalObjectReference{Name: s.credentials.Name})
 	}
 
-	hostAlias, err := s.prepareHostAlias()
+	hostAliases, err := s.getHostAliases()
 	if err != nil {
 		return err
 	}
 
-	podConfig := s.preparePodConfig(labels, annotations, podServices, imagePullSecrets, hostAlias, initContainers)
+	podConfig := s.preparePodConfig(labels, annotations, podServices, imagePullSecrets, hostAliases, initContainers)
 
 	s.Debugln("Creating build pod")
 	pod, err := s.kubeClient.CoreV1().Pods(s.configurationOverwrites.namespace).Create(&podConfig)
@@ -959,7 +923,7 @@ func (s *executor) preparePodConfig(
 	labels, annotations map[string]string,
 	services []api.Container,
 	imagePullSecrets []api.LocalObjectReference,
-	hostAlias *api.HostAlias,
+	hostAliases []api.HostAlias,
 	initContainers []api.Container,
 ) api.Pod {
 	buildImage := s.Build.GetAllVariables().ExpandValue(s.options.Image.Name)
@@ -1000,17 +964,24 @@ func (s *executor) preparePodConfig(
 			TerminationGracePeriodSeconds: &s.Config.Kubernetes.TerminationGracePeriodSeconds,
 			ImagePullSecrets:              imagePullSecrets,
 			SecurityContext:               s.Config.Kubernetes.GetPodSecurityContext(),
+			HostAliases:                   hostAliases,
 			Affinity:                      s.Config.Kubernetes.GetAffinity(),
+			DNSPolicy:                     s.getDNSPolicy(),
+			DNSConfig:                     s.Config.Kubernetes.GetDNSConfig(),
 		},
 	}
 
-	if hostAlias != nil {
-		pod.Spec.HostAliases = []api.HostAlias{*hostAlias}
-	}
-
 	return pod
 }
 
+func (s *executor) getDNSPolicy() api.DNSPolicy {
+	dnsPolicy, err := s.Config.Kubernetes.DNSPolicy.Get()
+	if err != nil {
+		s.Warningln(fmt.Sprintf("falling back to cluster's default policy: %v", err))
+	}
+	return dnsPolicy
+}
+
 func (s *executor) getHelperImage() string {
 	if len(s.Config.Kubernetes.HelperImage) > 0 {
 		return common.AppVersion.Variables().ExpandValue(s.Config.Kubernetes.HelperImage)
@@ -1278,19 +1249,10 @@ func (s *executor) checkDefaults() error {
 }
 
 func newExecutor() *executor {
-	helperImageInfo, err := helperimage.Get(common.REVISION, helperimage.Config{
-		OSType:       helperimage.OSTypeLinux,
-		Architecture: "amd64",
-	})
-	if err != nil {
-		logrus.WithError(err).Fatal("Failed to set up helper image for kubernetes executor")
-	}
-
 	e := &executor{
 		AbstractExecutor: executors.AbstractExecutor{
 			ExecutorOptions: executorOptions,
 		},
-		helperImageInfo:         helperImageInfo,
 		remoteProcessTerminated: make(chan shells.TrapCommandExitStatus),
 	}
 
