@@ -341,9 +341,10 @@ func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, build *common.Build,
 		b.writeGitSSLConfig(w, build, []string{"-f", templateFile})
 	}
 
+	b.writeGitCleanup(w, projectDir)
+
 	w.Command("git", "init", projectDir, "--template", templateDir)
 	w.Cd(projectDir)
-	b.writeGitCleanup(w)
 
 	// Add `git remote` or update existing
 	w.IfCmd("git", "remote", "add", "origin", build.GetRemoteURL())
@@ -363,14 +364,20 @@ func (b *AbstractShell) writeRefspecFetchCmd(w ShellWriter, build *common.Build,
 	w.Command("git", fetchArgs...)
 }
 
-func (b *AbstractShell) writeGitCleanup(w ShellWriter) {
-	// Remove .git/{index,shallow,HEAD}.lock files from .git, which can fail the fetch command
+func (b *AbstractShell) writeGitCleanup(w ShellWriter, projectDir string) {
+	// Remove .git/{index,shallow,HEAD,config}.lock files from .git, which can fail the fetch command
 	// The file can be left if previous build was terminated during git operation
-	w.RmFile(".git/index.lock")
-	w.RmFile(".git/shallow.lock")
-	w.RmFile(".git/HEAD.lock")
+	files := []string{
+		".git/index.lock",
+		".git/shallow.lock",
+		".git/HEAD.lock",
+		".git/hooks/post-checkout",
+		".git/config.lock",
+	}
 
-	w.RmFile(".git/hooks/post-checkout")
+	for _, f := range files {
+		w.RmFile(path.Join(projectDir, f))
+	}
 }
 
 func (b *AbstractShell) writeCheckoutCmd(w ShellWriter, build *common.Build) {
@@ -427,7 +434,6 @@ func (b *AbstractShell) writeSubmoduleUpdateCmd(w ShellWriter, build *common.Bui
 	}
 
 	// Clean changed files in submodules
-	// "git submodule update --force" option not supported in Git 1.7.1 (shipped with CentOS 6)
 	w.Command("git", append(foreachArgs, "git clean -ffxd")...)
 	w.Command("git", append(foreachArgs, "git reset --hard")...)
 	w.Command("git", updateArgs...)
